/*
 * Second attempt at a shared counter (and it is correct).
 * Threads alice and bob each increment the same counter
 * many times
 *
 * This version of the program avoids a data race on counter
 * by using mutual exclusion and synchronization. A single
 * mutex m guards access to counter. Only one person is allowed
 * to hold ("lock") the mutex at a given time. Other people
 * have to wait.
 *
 * Note that mutex lock/unlock are synchronizing operations, so
 * threads that call m.lock/unlock are communicating with each other
 * about changes to shared data, like counter (shared memory is weird!!).
 *
 * CSE 374, 2015-03 BM
 */

#include <thread>
#include <mutex>
#include <iostream>

using namespace std;

mutex m;

int counter = 0;

const int N = 1024*1024;

void alice_f() {
  for (int i=0; i<N; i++) {
    // wait until m is unlocked then lock it and proceed
    m.lock();

    // safely read and write the counter
    counter++;

    // unlock m so other threads may lock it
    m.unlock();
  }
}

void bob_f() {
  for (int i=0; i<N; i++) {
    m.lock();
    counter++;
    m.unlock();
  }
}

int main() {
  thread alice(alice_f);   
  thread bob(bob_f); 

  alice.join();
  bob.join();

  cout << "expected: " << N*2 << endl;
  cout << "actual: " << counter << endl;
  
}
